/*

   Temperature/Humidity Sensor with outputs stored in an EEPROM

   Hunter, Trevor, Jack, Andrew

*/

// standard Arduino libraries
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

// libraries for DHT and EEPROM
#include <DHT.h>
#include <SPI.h>
#include <W25Q16.h>

// set up necessary components for DHT and EEPROM
#define DHTPIN 5
#define DHTTYPE DHT22
DHT dht(DHTPIN, DHTTYPE);
W25Q16 flash;

// Graphical LCD commands
#define G_LCD_DATA               1                  // Data bit 
#define G_LCD_COMMAND            0                  // Command bit
#define G_LCD_CLEAR_SCREEN       0x01               // Clear screen
#define G_LCD_ADD_RESET          0x02               // Address counter is reset (restart cursor to 0x80 position) 
#define G_LCD_BASIC_FUNC         0x30               // Basic instruction set
#define G_LCD_EXTEND_FUNC        0x34               // Extended instruction set

// Graphical LCD Pins
#define G_LCD_RST  8                                // reset 
#define G_LCD_CS   10                               // chip select

// Joystick Pins
#define joyX A0
#define joyY A1
int button = 2;                                    // Pin to joystick button

// GUI Button Buffer
volatile uint8_t buttonRingBuff[16];               // power of 2 to use AND
volatile uint8_t B_inPtr;                          // next action of ring buffer
uint8_t B_outPtr;                                  // take value from ring buffer

// GUI Up/Down Buffer
volatile uint8_t Y_RingBuff[16];
volatile uint8_t Y_inPtr;
uint8_t Y_outPtr;
volatile uint8_t up_down;                          // 2 = down and 1 = up and 0 = reset
volatile uint8_t up_down_Last;

// GUI Right/Left Buffer
volatile uint8_t X_RingBuff[16];
volatile uint8_t X_inPtr;
uint8_t X_outPtr;
volatile uint8_t right_left;                       // 2 = left and 1 = right and 0 =  reset
volatile uint8_t right_left_Last;

// initialize all necessary pins and variables for EEPROM
int CS_MEMORY;
int DI_MEMORY;
int DO_MEMORY;
int CLK_MEMORY;

unsigned int startPage;
unsigned int startAddress;
unsigned int endPage;
unsigned int endAddress;
float percentRemainingMemory;

// declare TEMP variables
volatile float tempF = 0;
volatile float tempC = 0;
volatile float tempEEPROM = 0;
volatile float t = 0;

// declare BEAMBREAK variables
unsigned long start_time = 0;
unsigned long end_time = 0;
unsigned long flight_time = 0;
double state_1 = 0;
double state_2 = 0;

int STATE = 0;

// declare scale pins and variables
const byte DATA_PIN = A3;
const byte CLOCK_PIN = A4;
const byte GAIN = 1;
bool pinsConfigured;
volatile byte scale_tare = 0;

volatile float y1 = 50;                         // 50 grams is calibration mass
volatile long x1;
volatile long x0;
volatile float avg_size = 10.0;

// decalare select sensor variable
enum sensor {
  PH, PH_DATA, PH_LOG, SCALE, VOLT, TEMP, TEMP_DATA, TEMP_LOG, LINEAR, VERTICAL, NONE
};

volatile sensor sensor_type;
/*

   SETUP FUNCTIONS

*/

// Scale Functions
void HX711_init(uint8_t data_output_pin, uint8_t clock_pin) {       //set HX711 pins to MCU Registers, select gain
  uint8_t DATA_PIN = data_output_pin;
  uint8_t CLOCK_PIN = clock_pin;
  uint8_t GAIN = 1;
  bool pinsConfigured = true;
}

bool HX711_ReadytoSend() {
  if (!pinsConfigured) {                                            // Configure pins
    pinMode(DATA_PIN, INPUT);
    pinMode(CLOCK_PIN, OUTPUT);
    pinsConfigured = true;
  }
  return digitalRead(DATA_PIN) == LOW;                              // HX711 ready to send data when DATA_PIN is low
}

long Scale_read() {
  while (!HX711_ReadytoSend());                                      // Wait until ready to send data

  uint8_t data[3];                                                  // Create array of 4 bytes to hold data
  for (uint8_t j = 3; j--; ) {                                      // Shift data into array
    data[j] = shiftIn(DATA_PIN, CLOCK_PIN, MSBFIRST);
  }
  for (int i = 0; i < GAIN; i++) {                                  // set gain
    digitalWrite(CLOCK_PIN, HIGH);
    digitalWrite(CLOCK_PIN, LOW);
  }
  data[2] ^= 0x80;
  return ((uint32_t) data[2] << 16) | ((uint32_t) data[1] << 8) | (uint32_t) data[0];
}

void Scale_Tare() {
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);                   // clear screen
  delay(10);
  
  HX711_init(DATA_PIN, CLOCK_PIN);                                  // init scale
  delay(500);

  for (int i = 0; i < int(avg_size); i++) {
    delay(10);
    x0 += Scale_read();
  }

  x0 /= long(avg_size);

  G_LCD_Write(G_LCD_COMMAND, 0x80);         // Row 1
  const char *text1 = "Add Calibration";
  for (int i = 0; i < 15; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x90);         // Row 2
  const char *text2 = "Mass: 50 g";
  for (int i = 0; i < 10; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);


  int i = 1;
  while (1) {                                                       // Calibration
    if (Scale_read() < x0 + 1000) {
    } else {
      i++;
      delay(2000);
      for (int j = 0; j < int(avg_size); j++) {
        x1 += Scale_read();
      }
      x1 /= long(avg_size);
      break;
    }
  }
  Serial.println(scale_tare);
  G_LCD_Write(G_LCD_COMMAND, 0x80);         // Row 1
  const char *text3 = "Calibration    ";
  for (int i = 0; i < 15; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);

  G_LCD_Write(G_LCD_COMMAND, 0x90);         // Row 2
  const char *text4 = "Complete  ";
  for (int i = 0; i < 10; i++)
    G_LCD_Write(G_LCD_DATA, *text4++);

  delay(1000);                              // To Convey that calibration is complete
}

// *****************************
//    Graphical LCD Functions
// *****************************

void G_LCD_Write(boolean command, byte lcdData) {                               // Cursor Range: (lcdData of G_LCD_Write) 8 blocks of two digits
  SPI.beginTransaction(SPISettings(200000, MSBFIRST, SPI_MODE3));               // 1st Row: 0x80 to 0x87
  digitalWrite(G_LCD_CS, HIGH);                                                 // 2nd Row: 0x90 to 0x97
  SPI.transfer(command ? 0xFA : 0xF8);                                          // 3rd Row: 0x88 to 0x8F
  SPI.transfer(lcdData & 0xF0);                                                 // 4th Row: 0x98 to 0x9F
  SPI.transfer((lcdData << 4) & 0xF0);
  digitalWrite(G_LCD_CS, LOW);
  SPI.endTransaction();
}

void G_LCD_Init() {                                                             // LCD Intialization
  digitalWrite(G_LCD_RST, LOW);
  delay(100);
  digitalWrite(G_LCD_RST, HIGH);
  G_LCD_Write(G_LCD_COMMAND, G_LCD_BASIC_FUNC);                                 // Function set
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);                               // Display clear
  delay(10);
  G_LCD_Write(G_LCD_COMMAND, 0x06);                                             // Entry mode set (Cursors moves to right, increases address counter)
  delay(10);
  G_LCD_Write(G_LCD_COMMAND, 0x0C);                                             // Display control
}


// ** LCD Highlight Functions **
void G_LCD_SetGraphicMode(boolean enabled = true) {
  G_LCD_Write(G_LCD_COMMAND, enabled ? G_LCD_EXTEND_FUNC : G_LCD_BASIC_FUNC);
  if (enabled) G_LCD_Write(G_LCD_COMMAND, G_LCD_EXTEND_FUNC | 0x02);
}

void G_LCD_ClearGraphicMem() {
  for (byte x = 0; x < 16; x++)
    for (byte y = 0; y < 32; y++) {
      G_LCD_Write(G_LCD_COMMAND, 0x80 | y);
      G_LCD_Write(G_LCD_COMMAND, 0x80 | x);
      G_LCD_Write(G_LCD_DATA, 0x00);
      G_LCD_Write(G_LCD_DATA, 0x00);
    }
}

void G_LCD_Highlight(uint8_t input) {        // row = 1 through 4
  G_LCD_SetGraphicMode(true);
  G_LCD_ClearGraphicMem();
  uint16_t row;
  if (input == 1) {
    row = 0x80;
  } else if (input == 2) {
    row = 0x90;
  } else if (input == 3) {
    row = 0x88;
  } else {
    row = 0x98;
  }
  for (byte y = 0; y < 15; y++) {
    for (byte x = 0; x < 8; x++) {
      G_LCD_Write(G_LCD_COMMAND, row | y);
      G_LCD_Write(G_LCD_COMMAND, row | x);
      G_LCD_Write(G_LCD_DATA, 0xFF);
      G_LCD_Write(G_LCD_DATA, 0xFF);
    }
  }
  G_LCD_SetGraphicMode(false);
}

// ******************
//    GUI FUNCTIONS
// ******************

void GUI_Cursor(uint8_t row, uint8_t prev_row) {                              // Move * Cursor from one row to another
  switch (prev_row) {
    case 1:
      G_LCD_Write(G_LCD_COMMAND, 0x87);
      G_LCD_Write(G_LCD_DATA, 0x20);
      break;
    case 2:
      G_LCD_Write(G_LCD_COMMAND, 0x97);
      G_LCD_Write(G_LCD_DATA, 0x20);
      break;
    case 3:
      G_LCD_Write(G_LCD_COMMAND, 0x8F);
      G_LCD_Write(G_LCD_DATA, 0x20);
      break;
    default:
      break;
  }
  switch (row) {
    case 1:
      G_LCD_Write(G_LCD_COMMAND, 0x87);
      G_LCD_Write(G_LCD_DATA, 0x2A);
      break;
    case 2:
      G_LCD_Write(G_LCD_COMMAND, 0x97);
      G_LCD_Write(G_LCD_DATA, 0x2A);
      break;
    case 3:
      G_LCD_Write(G_LCD_COMMAND, 0x8F);
      G_LCD_Write(G_LCD_DATA, 0x2A);
      break;
    default:
      break;
  }
}

void GUI_Main_Menu() {                     // Main Menu Display
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x81);        // Row 1
  const char *text = "  MAIN MENU";
  for (int i = 0; i < 11; i++)
    G_LCD_Write(G_LCD_DATA, *text++);
  G_LCD_Highlight(0x1);

  G_LCD_Write(G_LCD_COMMAND, 0x91);        // Row 2
  const char *text2 = "Memory Menu";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
  GUI_Cursor(2, 0);

  G_LCD_Write(G_LCD_COMMAND, 0x89);        // Row 3
  const char *text3 = "Sensor Menu";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);
}

void GUI_Memory_Menu() {                   // Memory Menu Display
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Highlight(0x1);
  G_LCD_Write(G_LCD_COMMAND, 0x80);        // Row 1
  G_LCD_Write(G_LCD_DATA, 0x18);
  G_LCD_Write(G_LCD_COMMAND, 0x81);
  const char *text = "To Main Menu";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text++);

  G_LCD_Write(G_LCD_COMMAND, 0x91);        // Row 2
  const char *text2 = "Export Data";
  for (int i = 0; i < 11; i++) {
    G_LCD_Write(G_LCD_DATA, *text2++);
  }
  GUI_Cursor(2, 0);

  G_LCD_Write(G_LCD_COMMAND, 0x89);         // Row 3
  const char *text3 = "Clear  Data";
  for (int i = 0; i < 11; i++) {
    G_LCD_Write(G_LCD_DATA, *text3++);
  }

  G_LCD_Write(G_LCD_COMMAND, 0x98);        // Row 4
  const char *text4 = "Memory Used";
  for (int i = 0; i < 11; i++) {
    G_LCD_Write(G_LCD_DATA, *text4++);
  }
  
  G_LCD_Write(G_LCD_COMMAND, 0x9F);
  G_LCD_Write(G_LCD_DATA, 0x25);             //0x25 is %
  
  float data = 98.3;                         // write data to value of % memory remaining
  char outstr[2];                            // 2 digits 
  dtostrf(data, 2, 0, outstr);
  char *value = outstr;
  G_LCD_Write(G_LCD_COMMAND, 0x9E);         
  for (int i = 0; i < 2; i++)
    G_LCD_Write(G_LCD_DATA, *value++);
}

void GUI_Sensor_Main() {                     // First two rows on all sensor screens
  G_LCD_Write(G_LCD_COMMAND, 0x80);          // Row 1
  G_LCD_Write(G_LCD_DATA, 0x18);
  G_LCD_Write(G_LCD_COMMAND, 0x81);
  const char *text = "To Main Menu";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text++);
  G_LCD_Write(G_LCD_COMMAND, 0x87);
  G_LCD_Write(G_LCD_DATA, ' ');
  G_LCD_Write(G_LCD_DATA, 0x18);
  G_LCD_Highlight(0x1);

  G_LCD_Write(G_LCD_COMMAND, 0x90);          // Row 2
  const char *text2 = "<-            ->";
  for (int i = 0; i < 19; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
}

void GUI_Voltmeter() {                       // Voltmeter
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  Voltmeter   ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);

  G_LCD_Write(G_LCD_COMMAND, 0x98);          // Row 4
  const char *text2 = "              ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
}

void GUI_Scale() {                           // Scale
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  Scale       ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);
}

void GUI_PH() {
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  PH Sensor   ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);
}

void GUI_Temperature() {                     // Temperature
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = " Temperature";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x98);          // Row 4
  const char *text2 = "              ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
}

void GUI_LinDisplacement() {                 // Linear Displacement
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  Linear      ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);

  G_LCD_Write(G_LCD_COMMAND, 0x98);          // Row 4
  const char *text2 = "  Displacement";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
}

void GUI_VertDisplacement() {                // Vertical Displacement
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  Vertical    ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);

  G_LCD_Write(G_LCD_COMMAND, 0x98);          // Row 4
  const char *text2 = "  Displacement";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
}

void GUI_Output_Store() {                   // Output and Data Store
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x80);         // Row 1
  const char *text1 = "Output Data and";
  for (int i = 0; i < 16; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x90);         // Row 2
  const char *text2 = "Log Data";
  for (int i = 0; i < 8; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);

  G_LCD_Write(G_LCD_COMMAND, 0x8E);         // Row 3
  const char *text3 = "->";
  for (int i = 0; i < 2; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);

  G_LCD_Write(G_LCD_COMMAND, 0x9B);         // Row 4
  const char *text4 = "Yes";
  for (int i = 0; i < 3; i++)
    G_LCD_Write(G_LCD_DATA, *text4++);

  G_LCD_Write(G_LCD_COMMAND, 0x9D);
  G_LCD_Write(G_LCD_DATA, 0x2A);
}

void GUI_Output_Only() {                    // Output Data Only
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x80);         // Row 1
  const char *text1 = "Output Data Only";
  for (int i = 0; i < 16; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x89);         // Row 3
  const char *text2 = "<-";
  for (int i = 0; i < 2; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);

  G_LCD_Write(G_LCD_COMMAND, 0x9B);         // Row 4
  const char *text3 = "Yes";
  for (int i = 0; i < 3; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);

  G_LCD_Write(G_LCD_COMMAND, 0x9D);
  G_LCD_Write(G_LCD_DATA, 0x2A);
}

void GUI_Output_Stop() {                    // Data Output Screen and Stop Button
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x81);         // Row 1
  const char *text1 = "Data Output";
  for (int i = 0; i < 11; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x99);         // Row 4
  const char *text2 = "Stop";
  for (int i = 0; i < 4; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);

  G_LCD_Write(G_LCD_COMMAND, 0x9C);
  G_LCD_Write(G_LCD_DATA, 0x2A);

  G_LCD_Write(G_LCD_COMMAND, 0x9D);
  const char *text3 = "      ";
  for (int i = 0; i < 6; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);
}

void GUI_Export_Begin() {                   // Export Begin screen with start button
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x80);         // Row 1
  const char *text1 = "Plug Device Into";
  for (int i = 0; i < 16; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x90);         // Row 2
  const char *text2 = "Computer";
  for (int i = 0; i < 8; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);

  G_LCD_Write(G_LCD_COMMAND, 0x99);         // Row 4
  const char *text3 = "Export";
  for (int i = 0; i < 6; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);

  G_LCD_Write(G_LCD_COMMAND, 0x9D);
  G_LCD_Write(G_LCD_DATA, 0x2A);
}

void GUI_Clear_Check() {
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Highlight(0x1);
  G_LCD_Write(G_LCD_COMMAND, 0x80);         // Row 1
  G_LCD_Write(G_LCD_DATA, 0x18);
  G_LCD_Write(G_LCD_COMMAND, 0x81);
  const char *text = "To Main Menu";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text++);

  G_LCD_Write(G_LCD_COMMAND, 0x91);         // Row 2
  const char *text1 = "Are you sure?";
  for (int i = 0; i < 13; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x9A);         // Row 4
  const char *text2 = "Yes";
  for (int i = 0; i < 3; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);

  G_LCD_Write(G_LCD_COMMAND, 0x9C);
  G_LCD_Write(G_LCD_DATA, 0x2A);
}

void GUI_Plug_In_Sensor() {
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x80);         // Row 1
  const char *text1 = "Plug in Sensor";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x90);         // Row 2
  const char *text2 = "to the Device";
  for (int i = 0; i < 13; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);

  G_LCD_Write(G_LCD_COMMAND, 0x99);         // Row 4
  const char *text3 = "Ready";
  for (int i = 0; i < 5; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);

  G_LCD_Write(G_LCD_COMMAND, 0x9C);
  G_LCD_Write(G_LCD_DATA, 0x2A);
}

void Data_To_String_Output(float input, int input_l, int num_dec) {    // Takes data in float, number of digits of data, number of decimals in data
  float data = input;                                                  // and writes to screen at given row in funtion
  char outstr[input_l + 1];                                            // Takes unit in string and length of unit

  dtostrf(data, input_l, num_dec, outstr);
  char *value = outstr;
  G_LCD_Write(G_LCD_COMMAND, 0x89);         // Row 3
  for (int i = 0; i < input_l; i++)
    G_LCD_Write(G_LCD_DATA, *value++);
}


// declare pins and variables for EEPROM
void initEEPROM() {

  CS_MEMORY = 3;
  pinMode(CS_MEMORY, OUTPUT);
  DI_MEMORY = 11;
  DO_MEMORY = 12;
  CLK_MEMORY = 13;

  startPage = 0;
  startAddress = 0;
  endPage = 0;
  endAddress = 0;
}

// write values to EEPROM Memory Chip
void MEMORY_Write(uint32_t measurement) {

  // turn CS pin on
  flash.releasePowerDown();

  // write value to EEPROM memory
  flash.write(endPage, (byte)endAddress, (float)tempF);
  delay(50);
  endAddress = endAddress + 32;

  // check how much memory is remaining
  percentRemainingMemory = (1 - (endPage / 8191)) * 100;

  endAddress = startAddress;
  endPage = startPage;
  float tempEEPROM = flash.read(endPage, endAddress);
  endAddress = endAddress + 32;

  // print temp to Serial monitor from EEPROM
  Serial.print("EEPROM's temperature is: ");
  Serial.println(tempEEPROM);

  // increment page in memory if necessary
  if (endAddress >= 255) {
    endPage++;
    if (endPage > 8191) {
      endPage = 0;
    }
    endAddress = 0;
  }

  startPage = endPage;
  startAddress = endAddress;
  flash.powerDown();
}

// INTERRUPT ROUTINE //

ISR(TIMER1_COMPA_vect) {
  int xValue = analogRead(joyX);
  int yValue = analogRead(joyY);
  int buttonNow = digitalRead(2);
  int buttonLast = 1;

  if (buttonLast != buttonNow) {
    buttonLast = buttonNow;
    buttonRingBuff[B_inPtr] = buttonNow;
    B_inPtr++;
    B_inPtr &= 0x0F;
  }

  if (xValue >= 0 && yValue <= 10 ) {                  // up
    up_down = 1;
    if (up_down != up_down_Last) {
      Y_RingBuff[Y_inPtr] = 1;
      Y_inPtr++;
      Y_inPtr &= 0x0F;
      up_down_Last = up_down;                    // set up_down_Last to 0 after GUI movement in switch case
    } else {
      return;
    }
  } else if (xValue >= 300 && yValue >= 600) {        // down
    up_down = 2;
    if (up_down != up_down_Last) {
      Y_RingBuff[Y_inPtr] = 1;
      Y_inPtr--;
      Y_inPtr &= 0x0F;
      up_down_Last = up_down;                    // set up_down_Last to 0 after GUI movement in switch case
    } else {
      return;
    }

  } else if (xValue >= 600 && yValue >= 300) {        // right
    right_left = 1;
    if (right_left != right_left_Last) {
      X_RingBuff[X_inPtr] = 1;
      X_inPtr++;
      X_inPtr &= 0x0F;
      right_left_Last = right_left;              // set right_left_Last to 0 after GUI movement in switch case
    } else {
      return;
    }
  } else if (xValue <= 10 && yValue >= 300) {         // left
    right_left = 2;
    if (right_left != right_left_Last) {
      X_RingBuff[X_inPtr] = 1;
      X_inPtr--;
      X_inPtr &= 0x0F;
      right_left_Last = right_left;              // set right_left_Last to 0 after GUI movement in switch case
    } else {
      return;
    }
  } else {
    return;
  }
}

// Button Process (State Machine) Functions

void processYAction() {
  if (up_down == 1) {
    Y_outPtr++;
  } else {
    Y_outPtr--;
  }
  Y_outPtr &= 0x0F;

  switch (Y_outPtr) {
    case 0:                 // Sets Main menu at start up of device
      GUI_Main_Menu();
      B_outPtr = 0;
      B_inPtr = 0;
      up_down_Last = 0;
      break;
    case 15:                // Cursor Change in memory menu
      GUI_Cursor(3, 2);
      up_down_Last = 0;
      break;
    case 14:                // Return to Main Menu from Memory Menu,
      GUI_Main_Menu();
      B_outPtr = 0;
      B_inPtr = 0;
      Y_outPtr = 0;
      Y_inPtr = 0;
      up_down_Last = 0;
      break;
    case 13:                // set when button action = 1, cursor change in Memory_Menu
      GUI_Cursor(2, 3);
      up_down_Last = 0;
      break;
    case 12:                // Cursor change in Memory_Menu
      GUI_Cursor(3, 2);
      up_down_Last = 0;
      break;
    case 11:                // down boundary for Memory Menu
      GUI_Memory_Menu();
      Y_outPtr = 13;
      Y_inPtr = 13;
      up_down_Last = 0;
      break;
    case 1:                 // down boundary for Main Menu
      GUI_Main_Menu();
      B_outPtr = 0;
      B_inPtr = 0;
      Y_outPtr = 0;
      Y_inPtr = 0;
      up_down_Last = 0;
      break;
    default:
      up_down_Last = 0;
  }
  Serial.print("Y_action = ");
  Serial.println(Y_outPtr);
  Serial.println("----------");

}

void processXAction() {
  if (right_left == 1) {
    X_outPtr++;
  } else {
    X_outPtr--;
  }

  X_outPtr &= 0x0F;

  Serial.print("X_action = ");
  Serial.println(X_outPtr);
  Serial.println("----------");

  if (Y_outPtr == 15 & B_outPtr == 1) {   // in Sensor Menu and Output Type Menus
    switch (X_outPtr) {
      case 0:
        GUI_Voltmeter();
        right_left_Last = 0;
        break;
      case 1:
        GUI_LinDisplacement();
        right_left_Last = 0;
        break;
      case 2:
        GUI_VertDisplacement();
        right_left_Last = 0;
        break;
      case 3:
        GUI_Temperature();        // left boundary for sensor menu
        X_outPtr = 13;
        X_inPtr = 13;
        right_left_Last = 0;
        break;
      case 15:
        GUI_Scale();
        right_left_Last = 0;
        break;
      case 14:
        GUI_PH();
        right_left_Last = 0;
        break;
      case 13:
        GUI_Temperature();
        right_left_Last = 0;
        break;
      case 12:                    // right boundary for sensor menu
        GUI_VertDisplacement();
        X_outPtr = 2;
        X_inPtr = 2;
        right_left_Last = 0;
        break;
      default:
        right_left_Last = 0;
        break;
    }
  } else if (Y_outPtr == 15 & B_outPtr == 3) {
    switch (X_outPtr) {
      case 5:                     // left boundary for output and store GUI
        X_outPtr = 6;
        X_inPtr = 6;
        right_left_Last = 0;
        break;
      case 6:
        GUI_Output_Store();
        right_left_Last = 0;
        break;
      case 7:
        GUI_Output_Only();
        right_left_Last = 0;
        break;
      case 8:                    // right boundary for output only GUI
        X_outPtr = 7;
        X_inPtr = 7;
        right_left_Last = 0;
        break;
      default:
        right_left_Last = 0;
        break;
    }
  } else {                          // left or right movement in Data Output Screen
    return;
  }
}

void processButtonAction() {
  B_outPtr++;
  B_outPtr &= 0x0F;

  Serial.print("B_action= ");
  Serial.println(B_outPtr);
  Serial.println("----------");

  switch (B_outPtr) {
    case 1:
      if (Y_outPtr == 0) {                // Go to Memory Menu
        GUI_Memory_Menu();
        Y_outPtr = 13;
        Y_inPtr = 13;
        break;
      } else if (Y_outPtr == 15) {        // Go to Sensor Menu
        GUI_Sensor_Main();
        GUI_Voltmeter();
        X_outPtr = 0;
        X_inPtr = 0;
        break;
      } else {
        break;
      }
    case 2:                         // for button press on sensors X = 0,1,2,3,13,14,15
      if (Y_outPtr == 15) {
        switch (X_outPtr) {
          case 0:                   // Voltmeter
            GUI_Plug_In_Sensor();
            B_outPtr = 4;
            B_inPtr = 4;
            sensor_type = VOLT;
            break;
          case 1:                   // Linear Displacement
            GUI_Plug_In_Sensor();
            B_outPtr = 4;
            B_inPtr = 4;
            sensor_type = LINEAR;
            break;
          case 2:                   // Vertical Displacement
            GUI_Plug_In_Sensor();
            B_outPtr = 4;
            B_inPtr = 4;
            sensor_type = VERTICAL;
            break;
          case 13:                  // Temperature with Data Log
            GUI_Output_Store();
            X_outPtr = 6;
            X_inPtr = 6;
            B_outPtr = 3;
            B_inPtr = 3;
            sensor_type = TEMP;     // Set sensor enum
            break;
          case 14:                  // PH with Data Log option
            GUI_Output_Store();
            X_outPtr = 6;
            X_inPtr = 6;
            B_outPtr = 3;
            B_inPtr = 3;
            sensor_type = PH;
            break;
          case 15:                   // Scale
            GUI_Plug_In_Sensor();
            B_outPtr = 4;
            B_inPtr = 4;
            sensor_type = SCALE;
            break;
        }
      } else if (Y_outPtr == 13) {   // Go to Export Data Begin
        GUI_Export_Begin();
        break;
      }
      else if (Y_outPtr == 12) {    // Go to Check Clear Prompt
        GUI_Clear_Check();
        break;
      }
      else {
        break;
      }
    case 3:
      if (Y_outPtr == 13) {          // Export Data and go to Main Menu and clear states
        GUI_Main_Menu();
        B_outPtr = 0;
        B_inPtr = 0;
        X_outPtr = 0;
        X_inPtr = 0;
        Y_outPtr = 0;
        Y_inPtr = 0;
        break;
      } else if (Y_outPtr == 12) {    // Clear Memory and go to Main Menu
        // ENTER CLEAR MEMORY FUNCTION
        GUI_Main_Menu();
        B_outPtr = 0;
        B_inPtr = 0;
        X_outPtr = 0;
        X_inPtr = 0;
        Y_outPtr = 0;
        Y_inPtr = 0;
        break;
      } else {
        break;
      }
    case 4:                       // Prompt to Plug In Sensor
      if (Y_outPtr == 15) {
          if(sensor_type == TEMP){      // check for data and log or data only and set sensor type
            if(X_outPtr == 6){
              sensor_type = TEMP_LOG;
            } else if(X_outPtr == 7){
              sensor_type = TEMP_DATA;
            } else {break;}
          }
          if(sensor_type == PH){      // check for data and log or data only and set sensor type
            if(X_outPtr == 6){
              sensor_type = PH_LOG;
            } else if(X_outPtr == 7){
              sensor_type = PH_DATA;
            } else {break;}
          }         
        GUI_Plug_In_Sensor();
        break;
      } else {
        break;
      }
    case 5:                        // Display Data and give option to stop data and
      GUI_Output_Stop();
      break;
    case 6:                        // Stop data output from sensor and return to main menu and reset states
      GUI_Main_Menu();
      B_outPtr = 0;
      B_inPtr = 0;
      X_outPtr = 0;
      X_inPtr = 0;
      Y_outPtr = 0;
      Y_inPtr = 0;
      sensor_type = NONE; 
      scale_tare = 0; 
      break;
    default:
      break;
  }
}

/*

   MEASUREMENT FUNCTIONS

*/

// Select Sensor Output
void select_Sensor(sensor type) {
  switch (sensor_type) {
    case PH_DATA:
      break;
    case PH_LOG:
      break;
    case TEMP_DATA:
      measureTEMP();
      break;
    case TEMP_LOG:
      break;
    case SCALE:
      if(scale_tare == 0){        // set scale_tare back to 0 when data stopped in B_outPtr = 6
        Scale_Tare(); 
        scale_tare = 1;
        GUI_Output_Stop();
      }                          
      measureSCALE(); 
      break;
    case VOLT:
      break;
    case VERTICAL:
      break;
    case LINEAR:
      break;
    case NONE:
      break;
  }
}


// Digital Scale
void measureSCALE() {                                             // Call after tare to start outputting values
  long reading = 0;
  for (int i = 0; i < int(avg_size); i++) {
    reading += Scale_read();
  }
  reading /= long(avg_size);

  float ratio_1 = (float) (reading - x0);
  float ratio_2 = (float) (x1 - x0);
  float ratio = ratio_1 / ratio_2;
  float mass = y1 * ratio;

  if (mass <= 0) {
    mass = 0;
  }

  Data_To_String_Output(mass,6,2);
  
  G_LCD_Write(G_LCD_COMMAND, 0x8D);         // Row 3
  const char *text = "Grams";
    for (int i = 0; i < 5; i++)
    G_LCD_Write(G_LCD_DATA, *text++);
  
  Serial.print("Raw: ");
  Serial.print(reading);
  Serial.print(", ");
  Serial.println(mass);
}

// measure temperature (Fahrenheit and Celsius)
void measureTEMP() {
  tempC = dht.readTemperature();                          // average 10 measurements to ensure accuracy
  tempF = (tempC * 1.8) + 32;
  Data_To_String_Output(tempF, 5, 2);

  G_LCD_Write(G_LCD_COMMAND, 0x8D);         // Row 2
  G_LCD_Write(G_LCD_DATA, 0x46);            // 0x46 is "F"
}

void measureBEAMBREAK() {
  digitalWrite(6, HIGH); //turn on weak pull-up on digital input #6
  digitalWrite(7, HIGH); //turn on weak pull-up on digital input #7
  while (1) {

    //check states on both digital inputs
    state_1 = digitalRead(6);
    state_2 = digitalRead(7);


    if (state_1 == LOW) {
      start_time = millis();
    }
    else if (state_2 == LOW) {
      end_time = millis();
      flight_time = end_time - start_time;
      Serial.print("Time of Flight is: ");
      Serial.print((flight_time) / 1000.0, 3);
      Serial.println(" s");
      delay(200);
    }
  }
}

void measurePH() {

}

void measureVOLT(){
  
}

/*

   MAIN FUNCTION

*/
int main(void) {

  // initialize Arduino, DHT, and Serial
  init();
  cli();                                            // stops interrupts
  dht.begin();

  // initialize EEPROM and erase old content before powering down
  initEEPROM();
  flash.init(3);
  flash.chipErase();
  flash.powerDown();
  delay(10);

  pinMode(G_LCD_RST, OUTPUT);                       // LCD pin initialization
  SPI.begin();
  pinMode(G_LCD_CS, OUTPUT);
  digitalWrite(G_LCD_CS, LOW);                      // inactive

  // Button setup
  pinMode(button, INPUT);
  digitalWrite(button, HIGH);

  sensor_type = NONE;

  Serial.begin(9600);

  // TIMER1 interrupt
  TCCR1A = 0;                                      // set entire TCCR1A register to 0
  TCCR1B = 0;                                      // same for TCCR1B
  TCNT1  = 0;                                      // initialize counter value to 0
  // set compare match register for 100hz increments
  OCR1A = 2500;                                    // = (16*10^6) / (100*1024) - 1 (must be <65536)
  TCCR1B |= (1 << WGM12);                          // turn on CTC mode
  TCCR1B |= (1 << CS12) | (1 << CS10);             // Set CS12 and CS10 bits for 1024 prescaler
  TIMSK1 |= (1 << OCIE1A);                         // enable timer compare interrupt

  G_LCD_Init();
  GUI_Main_Menu();
  sei();                                          // allow interrupts

  // PRIMARY FUNCTION
  while (1) {

    // button presses & joystick
    if (B_inPtr != B_outPtr) {
      processButtonAction();
    }

    if (Y_inPtr != Y_outPtr) {
      processYAction();
    }

    if (X_inPtr != X_outPtr) {
      processXAction();
    }

    if (B_outPtr == 5) {
      select_Sensor(sensor_type);
    }
  }
}
