/*
   bitwise memory functions
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <SPI.h>

#define EE_CS        3   // EEPROM chip select                          (CS)
#define EE_DO        9   // EEPROM data out of EE in to processor       (DO)
#define EE_DI        12  // EEPROM data in to EE out from processor     (DI)
#define EE_CLK       4   // EEPROM clock                                (CLK)

volatile uint8_t eepromData = 0; // 8 bit value picked up from the eeprom
volatile uint32_t eepromAddress; // next EEPROM location to write to

uint8_t pageArray[256]; // array for storing measurements before shifting to eeprom
volatile uint8_t inPtr; // pointer that follows storing inside the array

uint8_t ticks;

// initialize 4 pins needed for EEPROM
void initEEPROM() {

  pinMode(EE_CS, OUTPUT);
  digitalWrite(EE_CS, HIGH);
  pinMode(EE_DO, INPUT); // DATA into processor from EEPROM
  pinMode(EE_DI, OUTPUT); // DATA into EEPROM from processor
  pinMode(EE_CLK, OUTPUT);

}

// functions to control chip select and write-protection
void CSLow() {
  digitalWrite(EE_CS, LOW);
}

void CSHigh() {
  digitalWrite(EE_CS, HIGH);
}

void writeEnable() {
  sendInstruction(0x06);
  CSHigh();
}

void writeDisable() {
  sendInstruction(0x04);
  CSHigh();
}

// send instruction byte to eeprom (ex. read, write enable, etc.)
void sendInstruction(uint8_t x) {
  CSLow();
  sendByte(x);
}

// function for a clock tick for EEPROM
void clkTick() {
  digitalWrite(EE_CLK, HIGH);   // high
  digitalWrite(EE_CLK, LOW);  // back low
}

// sends byte to the eeprom
void sendByte(uint8_t x) {
  uint8_t i;
  for (i = 0; i < 8; i++) {
    if (x & 0x80) {
      digitalWrite(EE_DI, HIGH);
    } else {
      digitalWrite(EE_DI, LOW);
    }
    clkTick();
    x = x << 1;
  }
}

// send a 24-bit address to the eeprom on DI pin
void sendAddress(uint32_t a) { // 24 bit value
  uint8_t i;
  a = a << 8;  // high bit in word is what we need
  for (i = 0; i < 24; i++) {
    if (a & 0x80000000) {
      digitalWrite(EE_DI, HIGH);
    } else {
      digitalWrite(EE_DI, LOW);
    }
    clkTick();
    a = a << 1;
  }
}

// write a byte to the eeprom on DI pin
void writeData(uint32_t a, uint8_t data) {
  writeEnable();
  sendInstruction(0x02); // page program instruction
  sendAddress(a);
  sendByte(data);
  CSHigh();
  writeDisable();
}

// read the byte on DO pin from the eeprom
uint8_t readByte() {
  uint8_t i;
  uint8_t r;
  uint8_t d;
  r = 0;
  
  for (i = 0; i < 8; i++) {
    d = digitalRead(EE_DO);
    r = r << 1;
    if (d == HIGH) { // get data bit
      r |= 0x01;
    }
    clkTick();
  }
  return r;
}

// read 16 bits on DO pin from the eeprom
uint16_t read16Bits() {
  uint8_t i;
  uint16_t r;
  r = 0;
  for (i = 0; i < 16; i++) {
    r = r << 1;
    if (EE_DO == LOW) { // get data bit
      r |= 0x01;
    }
    clkTick();
  }
  return r;
}

// read byte from a specific address
uint8_t readByteFromAddress(uint32_t a) {
  uint8_t r;
  r = 0;
  sendInstruction(0x03);
  sendAddress(a);
  r = readByte();
  return r;
}

/*

   READ/WRITE Functionality

*/

// function that writes a page on the specified address
void writePage(uint32_t a) {
  uint16_t i;
  writeEnable();
  sendInstruction(0x02);
  sendAddress(a);
  for (i = 0; i < 256; i++) {
    sendByte(pageArray[i]);   // iterate through pageArray writing one byte at a time
  }
  CSHigh();
  writeDisable();
}

// function that reads every byte on a page of the specified address
void readPage(uint32_t a) {
  uint16_t j;
  uint8_t n;
  eepromData = 0;
  
  sendInstruction(0x03);
  sendAddress(a);

  for (j = 0; j < 256; j++) { // every byte in page

    n = readByte();   // iterate through specified page reading one byte at a time
  }
  
  CSHigh();
}

// function erases the entire eeprom memory chip
void chipErase() {
  writeEnable();
  sendInstruction(0x60);
  CSHigh();
}

// function that checks the busy bit of Status Register 1 of the eeprom
uint8_t checkBusy() {
  uint8_t busy = 0;
  sendInstruction(0x05);
  busy = readByte();
  busy &= 0x01;
  return busy;
}

// function to get the device and manufacturer ID's from the chip and print it to Serial
void checkManuID() {
  writeEnable();
  sendInstruction(0x90);
  sendAddress(0x000000);
  
  uint8_t device = readByte();
  uint8_t manu = readByte();
    
  CSHigh();
  

  delay(1);
  Serial.print("Device ID is: ");
  Serial.println(device);
  Serial.print("Manufacturer ID is: ");
  Serial.println(manu);
  delay(1);
}

// function that finds the first available address within the eeprom
uint32_t findEepromAddress() {
  uint16_t checkPage = 0;
  uint32_t a = 0;
  volatile uint8_t flag = 0;
  while (flag == 0) {
    sendInstruction(0x03);
    sendAddress(a);
    checkPage = read16Bits();
    if (checkPage == 0xFFFF) {
      flag = 1;
    } else {
      a += 256;
      if (a > 0x07FFFFFF) { // ran out of pages, wrap around to page 0
        a = 0;
        flag = 1;
      }
    }
  }
  return a;
}

// erase the entire content of the memory chip
void eraseEEPROM() {
  readByteFromAddress(0x000000);  // dummy read
  chipErase();
  while(checkBusy() == 1){}   // wait until chipErase is complete
  eepromAddress = 0;
  delay(1);
}

// check the percentage of pages which have been filled
float percentMemoryRemaining(a) {
  percentMem = (1 - (a / 8192)) * 100;
}

int main(void) {
  init();
  initEEPROM();
  Serial.begin(115200);
  Serial.println("BEGIN");
  
  //eraseEEPROM();
  checkManuID();

  int i = 0;
  for (i = 0; i < 256; i++) {
    pageArray[i] = i; // fill buffer with 5's
  }

  while(1) {

    // write buffer to first page
//    writePage(0x000000);
//    delay(1);
//
//    // read first page and print values to Serial monitor
//    readPage(0x000000);
//    delay(10);

    writeEnable();
    sendInstruction(0x02);
    sendAddress(0x000000);
    sendByte(0x05);
    
    writeDisable();
    CSHigh();

    delay(1);

    readByteFromAddress(0x000000);

    delay(10);

  }

  return 0;
}

