#include <SPI.h>

// Graphical LCD commands
#define G_LCD_DATA               1                  // Data bit 
#define G_LCD_COMMAND            0                  // Command bit
#define G_LCD_CLEAR_SCREEN       0x01               // Clear screen
#define G_LCD_ADD_RESET          0x02               // Address counter is reset (restart cursor to 0x80 position) 
#define G_LCD_BASIC_FUNC         0x30               // Basic instruction set
#define G_LCD_EXTEND_FUNC        0x34               // Extended instruction set

// Graphical LCD Pins
#define G_LCD_RST  8                                // reset 
#define G_LCD_CS   10                               // chip select

// Joystick Pins
#define joyX A0
#define joyY A1
int button = 2;                                    // Pin to joystick button

// GUI Button Buffer
volatile uint8_t buttonRingBuff[16];               // power of 2 to use AND
volatile uint8_t B_inPtr;                          // next action of ring buffer
uint8_t B_outPtr;                                  // take value from ring buffer

// GUI Up/Down Buffer
volatile uint8_t Y_RingBuff[16];
volatile uint8_t Y_inPtr;
uint8_t Y_outPtr;
volatile uint8_t up_down;                          // 2 = down and 1 = up and 0 = reset
volatile uint8_t up_down_Last;

// GUI Right/Left Buffer
volatile uint8_t X_RingBuff[16];
volatile uint8_t X_inPtr;
uint8_t X_outPtr;
volatile uint8_t right_left;                       // 2 = left and 1 = right and 0 =  reset
volatile uint8_t right_left_Last;


// *****************************
//    Graphical LCD Functions
// *****************************

void G_LCD_Write(boolean command, byte lcdData) {                               // Cursor Range: (lcdData of G_LCD_Write) 8 blocks of two digits
  SPI.beginTransaction(SPISettings(200000, MSBFIRST, SPI_MODE3));               // 1st Row: 0x80 to 0x87
  digitalWrite(G_LCD_CS, HIGH);                                                 // 2nd Row: 0x90 to 0x97
  SPI.transfer(command ? 0xFA : 0xF8);                                          // 3rd Row: 0x88 to 0x8F
  SPI.transfer(lcdData & 0xF0);                                                 // 4th Row: 0x98 to 0x9F
  SPI.transfer((lcdData << 4) & 0xF0);
  digitalWrite(G_LCD_CS, LOW);
  SPI.endTransaction();
}

void G_LCD_Init() {                                                             // LCD Intialization 
  digitalWrite(G_LCD_RST, LOW);
  delay(100);
  digitalWrite(G_LCD_RST, HIGH);
  G_LCD_Write(G_LCD_COMMAND, G_LCD_BASIC_FUNC);                                 // Function set
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);                               // Display clear
  delay(10);
  G_LCD_Write(G_LCD_COMMAND, 0x06);                                             // Entry mode set (Cursors moves to right, increases address counter)
  delay(10);
  G_LCD_Write(G_LCD_COMMAND, 0x0C);                                             // Display control
}


// ** LCD Highlight Functions **
void G_LCD_SetGraphicMode(boolean enabled = true) {                            
  G_LCD_Write(G_LCD_COMMAND, enabled ? G_LCD_EXTEND_FUNC : G_LCD_BASIC_FUNC);
  if (enabled) G_LCD_Write(G_LCD_COMMAND, G_LCD_EXTEND_FUNC | 0x02);            
}

void G_LCD_ClearGraphicMem() {                                                  
  for (byte x = 0; x < 16; x++)
    for (byte y = 0; y < 32; y++) {
      G_LCD_Write(G_LCD_COMMAND, 0x80 | y);
      G_LCD_Write(G_LCD_COMMAND, 0x80 | x);
      G_LCD_Write(G_LCD_DATA, 0x00);
      G_LCD_Write(G_LCD_DATA, 0x00);}
}

void G_LCD_Highlight(uint8_t input) {        // row = 1 through 4
  G_LCD_SetGraphicMode(true);
  G_LCD_ClearGraphicMem();
  uint16_t row;
  if (input == 1) {
    row = 0x80;
  } else if (input == 2) {
    row = 0x90;
  } else if (input == 3) {
    row = 0x88;
  } else {
    row = 0x98;}
  for (byte y = 0; y < 15; y++) {
    for (byte x = 0; x < 8; x++) {
      G_LCD_Write(G_LCD_COMMAND, row | y);
      G_LCD_Write(G_LCD_COMMAND, row | x);
      G_LCD_Write(G_LCD_DATA, 0xFF);
      G_LCD_Write(G_LCD_DATA, 0xFF);
    }
  }
  G_LCD_SetGraphicMode(false);
}

// ******************
//    GUI FUNCTIONS
// ******************

void GUI_Cursor(uint8_t row, uint8_t prev_row) {                              // Move * Cursor from one row to another
  switch (prev_row) {
    case 1:
      G_LCD_Write(G_LCD_COMMAND, 0x87);
      G_LCD_Write(G_LCD_DATA, 0x20);
      break;
    case 2:
      G_LCD_Write(G_LCD_COMMAND, 0x97);
      G_LCD_Write(G_LCD_DATA, 0x20);
      break;
    case 3:
      G_LCD_Write(G_LCD_COMMAND, 0x8F);
      G_LCD_Write(G_LCD_DATA, 0x20);
      break;
    default:
      break;
  }
  switch (row) {
    case 1:
      G_LCD_Write(G_LCD_COMMAND, 0x87);
      G_LCD_Write(G_LCD_DATA, 0x2A);
      break;
    case 2:
      G_LCD_Write(G_LCD_COMMAND, 0x97);
      G_LCD_Write(G_LCD_DATA, 0x2A);
      break;
    case 3:
      G_LCD_Write(G_LCD_COMMAND, 0x8F);
      G_LCD_Write(G_LCD_DATA, 0x2A);
      break;
    default:
      break;
  }
}

void GUI_Main_Menu() {                     // Main Menu Display
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x81);        // Row 1
  const char *text = "  MAIN MENU";
  for (int i = 0; i < 11; i++)
    G_LCD_Write(G_LCD_DATA, *text++);
  G_LCD_Highlight(0x1);

  G_LCD_Write(G_LCD_COMMAND, 0x91);        // Row 2
  const char *text2 = "Memory Menu";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
  GUI_Cursor(2, 0);

  G_LCD_Write(G_LCD_COMMAND, 0x89);        // Row 3
  const char *text3 = "Sensor Menu";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);
}

void GUI_Memory_Menu() {                   // Memory Menu Display 
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN); 
  delay(10);

  G_LCD_Highlight(0x1);
  G_LCD_Write(G_LCD_COMMAND, 0x80);        // Row 1
  G_LCD_Write(G_LCD_DATA, 0x18);
  G_LCD_Write(G_LCD_COMMAND, 0x81);
  const char *text = "To Main Menu";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text++);

  G_LCD_Write(G_LCD_COMMAND, 0x91);        // Row 2
  const char *text2 = "Export Data";
  for (int i = 0; i < 11; i++) {
    G_LCD_Write(G_LCD_DATA, *text2++);
  }
  GUI_Cursor(2, 0);

  G_LCD_Write(G_LCD_COMMAND, 0x89);         // Row 3
  const char *text3 = "Clear  Data";
  for (int i = 0; i < 11; i++) {
    G_LCD_Write(G_LCD_DATA, *text3++);
  }

  G_LCD_Write(G_LCD_COMMAND, 0x98);
  const char *text4 = "Memory Used:";
  for (int i = 0; i < 12; i++) {
    G_LCD_Write(G_LCD_DATA, *text4++);
  }
}

void GUI_Sensor_Main() {                     // First two rows on all sensor screens
  G_LCD_Write(G_LCD_COMMAND, 0x80);          // Row 1
  G_LCD_Write(G_LCD_DATA, 0x18);
  G_LCD_Write(G_LCD_COMMAND, 0x81);
  const char *text = "To Main Menu";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text++);
  G_LCD_Write(G_LCD_COMMAND, 0x87);
  G_LCD_Write(G_LCD_DATA, ' ');
  G_LCD_Write(G_LCD_DATA, 0x18);
  G_LCD_Highlight(0x1);

  G_LCD_Write(G_LCD_COMMAND, 0x90);          // Row 2
  const char *text2 = "<-            ->";
  for (int i = 0; i < 19; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
}

void GUI_Voltmeter() {                       // Voltmeter
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  Voltmeter   ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);
}

void GUI_Scale() {                           // Scale
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  Scale       ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);
}

void GUI_PH() {
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  PH Sensor   ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);
}

void GUI_Temperature() {                     // Temperature
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = " Temperature";
  for (int i = 0; i < 12; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x98);          // Row 4
  const char *text2 = "              ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
}

void GUI_Ammeter() {                         // Ammeter
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  Ammeter     ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);

  G_LCD_Write(G_LCD_COMMAND, 0x98);          // Row 4
  const char *text2 = "              ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
}

void GUI_LinDisplacement() {                 // Linear Displacement
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  Linear      ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);

  G_LCD_Write(G_LCD_COMMAND, 0x98);          // Row 4
  const char *text2 = "  Displacement";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
}

void GUI_VertDisplacement() {                // Vertical Displacement
  G_LCD_Write(G_LCD_COMMAND, 0x88);          // Row 3
  const char *text1 = "  Vertical    ";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);
  GUI_Cursor(3, 0);

  G_LCD_Write(G_LCD_COMMAND, 0x98);          // Row 4
  const char *text2 = "  Displacement";
  for (int i = 0; i < 14; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);
}

void GUI_Output_Store() {                   // Output and Data Store
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x80);         // Row 1
  const char *text1 = "Output Data and";
  for (int i = 0; i < 16; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x90);         // Row 2
  const char *text2 = "Log Data";
  for (int i = 0; i < 8; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);

  G_LCD_Write(G_LCD_COMMAND, 0x8E);         // Row 3
  const char *text3 = "->";
  for (int i = 0; i < 2; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);

  G_LCD_Write(G_LCD_COMMAND, 0x9B);         // Row 4
  const char *text4 = "Yes";
  for (int i = 0; i < 3; i++)
    G_LCD_Write(G_LCD_DATA, *text4++);

  G_LCD_Write(G_LCD_COMMAND, 0x9D);
  G_LCD_Write(G_LCD_DATA, 0x2A);
}
  
void GUI_Output_Only() {                    // Output Data Only
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x80);         // Row 1
  const char *text1 = "Output Data Only";
  for (int i = 0; i < 16; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x89);         // Row 3
  const char *text2 = "<-";
  for (int i = 0; i < 2; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);

  G_LCD_Write(G_LCD_COMMAND, 0x9B);         // Row 4
  const char *text3 = "Yes";
  for (int i = 0; i < 3; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);

  G_LCD_Write(G_LCD_COMMAND, 0x9D);
  G_LCD_Write(G_LCD_DATA, 0x2A);
}

void GUI_Output_Stop() {                    // Data Output Screen and Stop Button
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x81);         // Row 1
  const char *text1 = "Data Output";
  for (int i = 0; i < 11; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  // INSERT CODE FOR DATA OUTPUT            // Row 2

  G_LCD_Write(G_LCD_COMMAND, 0x99);         // Row 4
  const char *text2 = "Stop";
  for (int i = 0; i < 4; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);

  G_LCD_Write(G_LCD_COMMAND, 0x9C);
  G_LCD_Write(G_LCD_DATA, 0x2A);
}

void GUI_Export_Begin() {                   // Export Begin screen with start button
  G_LCD_Write(G_LCD_COMMAND, G_LCD_CLEAR_SCREEN);
  delay(10);

  G_LCD_Write(G_LCD_COMMAND, 0x80);         // Row 1
  const char *text1 = "Plug Device Into";
  for (int i = 0; i < 16; i++)
    G_LCD_Write(G_LCD_DATA, *text1++);

  G_LCD_Write(G_LCD_COMMAND, 0x90);         // Row 2
  const char *text2 = "Computer";
  for (int i = 0; i < 8; i++)
    G_LCD_Write(G_LCD_DATA, *text2++);

  G_LCD_Write(G_LCD_COMMAND, 0x99);         // Row 4
  const char *text3 = "Export";
  for (int i = 0; i < 6; i++)
    G_LCD_Write(G_LCD_DATA, *text3++);

  G_LCD_Write(G_LCD_COMMAND, 0x9D);
  G_LCD_Write(G_LCD_DATA, 0x2A);
}



// INTERRUPT ROUTINE //

ISR(TIMER1_COMPA_vect) {
  int xValue = analogRead(joyX);
  int yValue = analogRead(joyY);
  int buttonNow = digitalRead(2);
  int buttonLast = 1;

  if (buttonLast != buttonNow) {
    buttonLast = buttonNow;
    buttonRingBuff[B_inPtr] = buttonNow;
    B_inPtr++;
    B_inPtr &= 0x0F;
  }

  if (xValue >= 0 && yValue <= 10 ) {                  // up
    up_down = 1;
    if (up_down != up_down_Last) {
      Y_RingBuff[Y_inPtr] = 1;
      Y_inPtr++;
      Y_inPtr &= 0x0F;
      up_down_Last = up_down;                    // set up_down_Last to 0 after GUI movement in switch case
    } else {
      return;
    }
  } else if (xValue >= 300 && yValue >= 600) {        // down
    up_down = 2;
    if (up_down != up_down_Last) {
      Y_RingBuff[Y_inPtr] = 1;
      Y_inPtr--;
      Y_inPtr &= 0x0F;
      up_down_Last = up_down;                    // set up_down_Last to 0 after GUI movement in switch case
    } else {
      return;
    }

  } else if (xValue >= 600 && yValue >= 300) {        // right
    right_left = 1;
    if (right_left != right_left_Last) {
      X_RingBuff[X_inPtr] = 1;
      X_inPtr++;
      X_inPtr &= 0x0F;
      right_left_Last = right_left;              // set right_left_Last to 0 after GUI movement in switch case
    } else {
      return;
    }
  } else if (xValue <= 10 && yValue >= 300) {         // left
    right_left = 2;
    if (right_left != right_left_Last) {
      X_RingBuff[X_inPtr] = 1;
      X_inPtr--;
      X_inPtr &= 0x0F;
      right_left_Last = right_left;              // set right_left_Last to 0 after GUI movement in switch case
    } else {
      return;
    }
  } else {
    return;
  }
}

// Button Process (State Machine) Functions

void processYAction() {
  if (up_down == 1) {
    Y_outPtr++;
  } else {
    Y_outPtr--;
  }
  Y_outPtr &= 0x0F;
  
  switch (Y_outPtr) {
    case 0:                 // Sets Main menu at start up of device
      GUI_Main_Menu();
      B_outPtr = 0;
      B_inPtr = 0;
      up_down_Last = 0;
      break;
    case 15:                // Cursor Change in memory menu
      GUI_Cursor(3, 2);
      up_down_Last = 0;
      break;
    case 14:                // Return to Main Menu from Memory Menu,
      GUI_Main_Menu();
      B_outPtr = 0;
      B_inPtr = 0;
      Y_outPtr = 0;
      Y_inPtr = 0;
      up_down_Last = 0;
      break;
    case 13:                // set when button action = 1, cursor change in Memory_Menu
      GUI_Cursor(2, 3);
      up_down_Last = 0;
      break;
    case 12:                // Cursor change in Memory_Menu
      GUI_Cursor(3, 2);
      up_down_Last = 0;
      break;
    case 11:                // down boundary for Memory Menu
      GUI_Memory_Menu();
      Y_outPtr = 13;
      Y_inPtr = 13;
      up_down_Last = 0;
      break;
    case 1:                 // down boundary for Main Menu
      GUI_Main_Menu();
      B_outPtr = 0;
      B_inPtr = 0;
      Y_outPtr = 0;
      Y_inPtr = 0;
      up_down_Last = 0;
      break;
    default:
      up_down_Last = 0;
  }
  Serial.print("Y_action = ");
  Serial.println(Y_outPtr);
  Serial.println("----------");

}

void processXAction() {
  if (right_left == 1) {
    X_outPtr++;
  } else {
    X_outPtr--;
  }

  X_outPtr &= 0x0F;

  Serial.print("X_action = ");
  Serial.println(X_outPtr);
  Serial.println("----------");

  if (Y_outPtr == 15 & B_outPtr == 1) {   // in Sensor Menu and Output Type Menus
    switch (X_outPtr) {
      case 1:
        GUI_Ammeter();
        right_left_Last = 0;
        break;
      case 2:
        GUI_LinDisplacement();
        right_left_Last = 0;
        break;
      case 3:
        GUI_VertDisplacement();
        right_left_Last = 0;
        break;
      case 4:
        GUI_Temperature();
        X_outPtr = 13;
        X_inPtr = 13;
        right_left_Last = 0;
        break;
      case 15:
        GUI_Scale();
        right_left_Last = 0;
        break;
      case 14:
        GUI_PH();
        right_left_Last = 0;
        break;
      case 13:
        GUI_Temperature();
        right_left_Last = 0;
        break;
      case 12:
        GUI_VertDisplacement();
        X_outPtr = 3;
        X_inPtr = 3;
        right_left_Last = 0;
        break;
      default:
        right_left_Last = 0;
        break;
    }
  } else if (Y_outPtr == 15 & B_outPtr == 3) {
    switch (X_outPtr) {
      case 5:                     // left boundary for output and store GUI
        X_outPtr = 6;
        X_inPtr = 6;
        right_left_Last = 0;
        break;
      case 6:
        GUI_Output_Store();
        right_left_Last = 0;
        break;
      case 7:
        GUI_Output_Only();
        right_left_Last = 0;
        break;
      case 8:                    // right boundary for output only GUI
        X_outPtr = 7;
        X_inPtr = 7;
        right_left_Last = 0;
        break;
      default:
        right_left_Last = 0;
        break;
    }
  } else {                          // left or right movement in Data Output Screen
    return;
  }
}

void processButtonAction() {
  B_outPtr++;
  B_outPtr &= 0x0F;

  Serial.print("B_action= ");
  Serial.println(B_outPtr);
  Serial.println("----------");

  switch (B_outPtr) {
    case 1:
      if (Y_outPtr == 0) {
        GUI_Memory_Menu();
        Y_outPtr = 13;
        Y_inPtr = 13;
        break;
      } else if (Y_outPtr == 15) {
        GUI_Sensor_Main();
        GUI_Voltmeter();
        X_outPtr = 0;
        X_inPtr = 0;
        break;
      } else {
        break;
      }
    case 2:                   // for button press on sensors X = 0,1,2,3,13,14,15
      if (Y_outPtr == 15) {
        switch (X_outPtr) {
          case 0:                   // Voltmeter
            GUI_Output_Store();
            break;
          case 1:                   // Ammeter
            GUI_Output_Store();
            break;
          case 2:                   // Linear Displacement
            GUI_Output_Store();
            break;
          case 3:                   // Vertical Displacement
            GUI_Output_Store();
            break;
          case 13:                  // Temperature with Data Log option
            GUI_Output_Store();
            X_outPtr = 6;
            X_inPtr = 6;
            B_outPtr = 3;
            B_inPtr = 3;
            break;
          case 14:                  // PH with Data Log option
            GUI_Output_Store();
            X_outPtr = 6;
            X_inPtr = 6;
            B_outPtr = 3;
            B_inPtr = 3;
            break;
          case 15:                  // Scale
            GUI_Output_Store();
            break;
        }
      } else if (Y_outPtr == 13) {   // Go to Export Data Begin
        GUI_Export_Begin();
        break;
      }
      else if (Y_outPtr == 12) {    // Clear Data Chip and go to Main Menu and clear states
        GUI_Main_Menu();
        B_outPtr = 0;
        B_inPtr = 0;
        X_outPtr = 0;
        X_inPtr = 0;
        Y_outPtr = 0;
        Y_inPtr = 0;
        break;
      }
      else {
        break;
      }
    case 3:
      if (Y_outPtr == 13) {          // Export Data and go to Main Menu and clear states
        GUI_Main_Menu();
        B_outPtr = 0;
        B_inPtr = 0;
        X_outPtr = 0;
        X_inPtr = 0;
        Y_outPtr = 0;
        Y_inPtr = 0;
        break;
      }  else {
        break;
      }
    case 4:
      if (Y_outPtr == 15) {         // Display Data and give option to stop data and
        GUI_Output_Stop();
        break;
      } else {
        break;
      }
    case 5:                        // Stop data output from sensor and return to main menu and reset states
      GUI_Main_Menu();
      B_outPtr = 0;
      B_inPtr = 0;
      X_outPtr = 0;
      X_inPtr = 0;
      Y_outPtr = 0;
      Y_inPtr = 0;
      break;
    default:
      break;
  }
}

void setup() {
  pinMode(G_LCD_RST, OUTPUT);
  SPI.begin();
  pinMode(G_LCD_CS, OUTPUT);
  digitalWrite(G_LCD_CS, LOW);                      // inactive

  Serial.begin(9600);

  // Joystick Interrupt
  cli();                                           // stops interrupts
  TCCR1A = 0;                                      // set entire TCCR1A register to 0
  TCCR1B = 0;                                      // same for TCCR1B
  TCNT1  = 0;                                      // initialize counter value to 0
  // set compare match register for 100hz increments
  OCR1A = 2500;                                    // = (16*10^6) / (100*1024) - 1 (must be <65536)
  TCCR1B |= (1 << WGM12);                          // turn on CTC mode
  TCCR1B |= (1 << CS12) | (1 << CS10);             // Set CS12 and CS10 bits for 1024 prescaler
  TIMSK1 |= (1 << OCIE1A);                         // enable timer compare interrupt

  // Button setup
  pinMode(button, INPUT);
  digitalWrite(button, HIGH);

  G_LCD_Init();
  GUI_Main_Menu();
  sei();                                          // allow interrupts
}

void loop() {

  if (B_inPtr != B_outPtr) {
    processButtonAction();
  }

  if (Y_inPtr != Y_outPtr) {
    processYAction();
  }

  if (X_inPtr != X_outPtr) {
    processXAction();
  }
}
