/*
 * Temperature/Humidity Sensor with outputs stored in an EEPROM
 */

// standard Arduino libraries
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

// libraries for DHT and EEPROM
#include <DHT.h>
#include <SPI.h>
#include <W25Q16.h>

// set up necessary components for DHT and EEPROM
#define DHTPIN 11
#define DHTTYPE DHT22   
DHT dht(DHTPIN, DHTTYPE); 
W25Q16 flash;

// initialize all necessary pins and variables
int CS_MEMORY;
int DI_MEMORY;
int DO_MEMORY;
int CLK_MEMORY;

volatile unsigned int startPage;
volatile unsigned int startAddress;
volatile unsigned int endPage;
volatile unsigned int endAddress;
volatile float percentRemainingMemory;

/*
 * FUNCTIONS TO BE CALLED HERE
 */

// function to initialize pins for EEPROM
void initEEPROM() {
  
  CS_MEMORY = 10;
  DI_MEMORY = 12;
  DO_MEMORY = 11;
  CLK_MEMORY = 13;

  startPage = 0;
  startAddress = 0;
  endPage = 0;
  endAddress = 0;
}

// function to check page status of EEPROM
void pageEnd() {
  if (endAddress>255) {
    endPage++;
    if (endPage>8191) {
      //endPage = 0;
      Serial.println("ERROR: Out of memory!");
      return;
    }
    endAddress = 0;
  }
}

// function to check % memory used
void checkRemainingMemory() {
  percentRemainingMemory = (1 - (endPage / 8191)) * 100;
}

/*
 * MAIN FUNCTION HERE
 */

int main(void) {

  // initialize Arduino, DHT, and Serial
  init();
  dht.begin();
  Serial.begin(115200);

  // initialize EEPROM
  initEEPROM();
  flash.init(10);
  flash.chipErase();

  // initialize variables
  volatile float temp;
  volatile float humid;
  volatile float tempEEPROM;
  volatile float humidEEPROM;

  // ensure EEPROM is initialized properly
  byte manID = flash.manufacturerID(); 
  Serial.print("Manufacturer ID: ");
  Serial.println(manID,HEX);

  flash.powerDown(); 
  int i=0;

  // primary function
  while(i!=1) {

    //perform measurements every second
    flash.releasePowerDown();
    delay(1000);
    
    // measure temp and humidity
    temp = dht.readTemperature();
    humid = dht.readHumidity();

    // check if temp and humidity measurements are good
    if(isnan(temp) || isnan(humid)) {
      Serial.println("FAIL");
      return;
    }

    // convert temp from Celsius into Fahrenheit
    temp = (temp * (9/5)) + 32;

    // print temp and humidity to Serial monitor
    Serial.print("temperature is: ");
    Serial.println(temp);
    Serial.print("humidity is: ");
    Serial.println(humid);

    // write temp and humidity to EEPROM memory
    flash.write(endPage, endAddress, temp);
    endAddress++;
    pageEnd();    // check if end of page has been reached
    flash.write(endPage, endAddress, humid);
    endAddress++;

    // read from EEPROM and print to Serial monitor
    endAddress = startAddress;
    endPage = startPage;
    tempEEPROM = flash.read(endPage, endAddress);
    endAddress++;
    humidEEPROM = flash.read(endPage, endAddress);

    // print temp and humidity to Serial monitor from EEPROM
    Serial.print("EEPROM's temperature is: ");
    Serial.println(tempEEPROM);
    Serial.print("EEPROM's humidity is: ");
    Serial.println(humidEEPROM);

    checkRemainingMemory();
    Serial.print("% remaining memory is: ");
    Serial.print(percentRemainingMemory);
    Serial.println("%");

    i = 1;
    flash.powerDown();
    startPage = endPage;
    startAddress = endAddress;
    delay(100);
  }
}
